# This file is a part of datanommer, a message sink for fedmsg.
# Copyright (C) 2014, Red Hat, Inc.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#!/usr/bin/env python
""" This script just emits a bunch of random messages on an endpoint to which
busmon is listening.  Run it while busmon is running to provide it with fake
test data.

    :author: Ralph Bean <rbean@redhat.com>

"""

import random
import time
import simplejson

import fedmsg


def main():
    # Prepare our context and publisher
    fedmsg.init(name="bodhi.marat")

    # Probabilities of us emitting an event on each topic.
    probs = {
        'bodhi': 0.35,
        'fedoratagger': 0.2,
        'pkgdb': 0.1,
        'fas': 0.2,
        'mediawiki': 0.3,
    }

    # Main loop
    i = 0
    while True:
        for service, thresh in probs.iteritems():
            if random.random() < thresh:
                print service, thresh
                fedmsg.send_message(
                    topic='fake_data',
                    msg={'test': "Test data." + str(i)},
                    modname=service,
                )
                i = i + 1
        time.sleep(random.random())

if __name__ == "__main__":
    main()



# This file is a part of datanommer, a message sink for fedmsg.
# Copyright (C) 2014, Red Hat, Inc.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
import fedmsg
import fedmsg.consumers
import datanommer.models

DEFAULTS = {
    'datanommer.enabled': False,
    # Put a sqlite db in the current working directory if the user doesn't
    # specify a real location.
    'datanommer.sqlalchemy.url': 'sqlite:///datanommer.db',
}


import logging
log = logging.getLogger("fedmsg")


class Nommer(fedmsg.consumers.FedmsgConsumer):
    topic = "*"
    config_key = 'datanommer.enabled'

    def __init__(self, hub):
        # The superclass __init__() subscribes the hub to the topic specified
        # by the consumer. If we have a topic we want use instead of "*", it
        # needs to be set before calling the superclass.
        if 'datanommer.topic' in hub.config:
            self.topic = hub.config['datanommer.topic']

        super(Nommer, self).__init__(hub)

        # If fedmsg doesn't think we should be enabled, then we should quit
        # before setting up all the extra special zmq machinery.
        # _initialized is set in moksha.api.hub.consumer
        if not getattr(self, "_initialized", False):
            return

        # Setup a sqlalchemy DB connection (postgres, or sqlite)
        datanommer.models.init(self.hub.config['datanommer.sqlalchemy.url'])

    def consume(self, message):
        log.debug("Nomming %r" % message)
        try:
            datanommer.models.add(message)
        except Exception:
            datanommer.models.session.rollback()
            raise



# This file is a part of datanommer, a message sink for fedmsg.
# Copyright (C) 2014, Red Hat, Inc.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
from setuptools import setup, find_packages
import sys

f = open('README.rst')
long_description = f.read().strip()
long_description = long_description.split('split here', 1)[1]
f.close()

version = '0.8.0'

setup(
    name='datanommer.consumer',
    version=version,
    description="Hub consumer plugin for datanommer",
    long_description=long_description,
    author='Ralph Bean',
    author_email='rbean@redhat.com',
    url='http://github.com/fedora-infra/datanommer',
    license='GPLv3+',
    packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
    namespace_packages=['datanommer'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        "datanommer.models",
        "fedmsg",
    ],
    entry_points={
        'moksha.consumer': (
            'noms = datanommer.consumer:Nommer'
        ),
    },
    tests_require=[
        "nose",
        "mock",
        "psutil",
        "moksha.hub",
    ],
    test_suite='nose.collector',
)



# This file is a part of datanommer, a message sink for fedmsg.
# Copyright (C) 2014, Red Hat, Inc.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
import unittest
import mock
import copy

from sqlalchemy.exc import IntegrityError
from sqlalchemy.orm import scoped_session

from nose.tools import eq_

import datanommer.consumer
import datanommer.models

import os

filename = ":memory:"


class TestConsumer(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        import fedmsg.config
        import fedmsg.meta

        config = fedmsg.config.load_config([], None)
        uri = "sqlite:///%s" % filename
        config['datanommer.sqlalchemy.url'] = uri
        config['datanommer.enabled'] = True
        fedmsg.meta.make_processors(**config)
        cls.fedmsg_config = config

    def setUp(self):
        class FakeHub(object):
            config = self.fedmsg_config

            def subscribe(*args, **kwargs):
                pass

        # We only have to do this so that we can do it over
        # and over again for each test.
        datanommer.models.session = scoped_session(datanommer.models.maker)
        datanommer.models.init(
            self.fedmsg_config['datanommer.sqlalchemy.url'],
            create=True,
        )
        datanommer.consumer.Nommer._initialized = True  # Clearly, a lie.
        self.consumer = datanommer.consumer.Nommer(FakeHub())

    def tearDown(self):
        engine = datanommer.models.session.get_bind()
        datanommer.models.DeclarativeBase.metadata.drop_all(engine)
        datanommer.models._users_seen = set()
        datanommer.models._packages_seen = set()

    def test_duplicate_msg_id(self):
        example_message = dict(
            topic="topiclol",
            body=dict(
                topic="topiclol",
                i=1,
                msg_id="1234",
                timestamp=1234,
                msg=dict(
                    foo="bar",
                )
            )
        )
        msg1 = copy.deepcopy(example_message)
        msg2 = copy.deepcopy(example_message)

        self.consumer.consume(msg1)
        eq_(datanommer.models.Message.query.count(), 1)

        with mock.patch("fedmsg.publish") as mocked_function:
            with self.assertRaises(IntegrityError):
                # The database layer should raise an exception on
                # the unique constraint violation.
                self.consumer.consume(msg2)
            eq_(datanommer.models.Message.query.count(), 1)

        mocked_function.assert_not_called()
